--------------------------------------------------------------------------------
-- Bunker Buster, Holzchopf, 2011
-- v2.1 - 28 Sep 2011	- Updated hierarchy
-- v2.0 - 07 Jul 2011	- Explosion radius decreased
--						- Damage decreased
--						- Projectile velocity decreased
--						- Once ignited, the rocket can't drown anymore
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.bunkerbuster={}
holzchopf.bunkerbuster.rocket={}

-- load ressources
holzchopf.bunkerbuster.gfx_projw=loadgfx("holzchopf/bunkerbusterw.png")
setmidhandle(holzchopf.bunkerbuster.gfx_projw)
holzchopf.bunkerbuster.gfx_proj=loadgfx("holzchopf/bunkerbuster.png")
setmidhandle(holzchopf.bunkerbuster.gfx_proj)
holzchopf.bunkerbuster.gfx_icn=loadgfx("holzchopf/bunkerbustericon.png")
setmidhandle(holzchopf.bunkerbuster.gfx_icn)

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.bunkerbuster.id=addweapon("holzchopf.bunkerbuster","Bunker Buster",holzchopf.bunkerbuster.gfx_icn,0,3)

function holzchopf.bunkerbuster.draw()
	-- HUD Positioning & Timer
	if weapon_shots==0 then
		hudpositioning(pos_invisible)
		hudinfo('Set the time from first impact to detonation in tenths of seconds')
		hudtimer(2,2,10)
	end
end

function holzchopf.bunkerbuster.attack(attack)
	if (weapon_shots<=0) and (weapon_position==1) then
		-- No more weapon switching!
		useweapon(0)
		weapon_shots=weapon_shots+1
		-- Attack
		pid=createprojectile(holzchopf.bunkerbuster.rocket.id)
		projectiles[pid]={}
		projectiles[pid].rot=180-getwind()*200
		projectiles[pid].y=-1000
		projectiles[pid].x=weapon_x+math.tan(math.rad(projectiles[pid].rot))*(weapon_y-projectiles[pid].y)
		projectiles[pid].sx=math.sin(math.rad(projectiles[pid].rot))*15.0
		projectiles[pid].sy=-math.cos(math.rad(projectiles[pid].rot))*15.0
		projectiles[pid].state=0
		projectiles[pid].drown=0
		-- End Turn
		endturn()
	end
end

--------------------------------------------------------------------------------
-- Rocket
--------------------------------------------------------------------------------
holzchopf.bunkerbuster.rocket.id=addprojectile("holzchopf.bunkerbuster.rocket")

function holzchopf.bunkerbuster.rocket.draw(id)
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(projectiles[id].rot)
	-- Draw projectile
	if projectiles[id].state==0 then
		drawimage(holzchopf.bunkerbuster.gfx_projw,projectiles[id].x,projectiles[id].y)
	else
		drawimage(holzchopf.bunkerbuster.gfx_proj,projectiles[id].x,projectiles[id].y)
	end
end

function holzchopf.bunkerbuster.rocket.update(id)
	-- Particle Tail
	particle(p_smoke,projectiles[id].x-math.sin(math.rad(projectiles[id].rot))*7,projectiles[id].y+math.cos(math.rad(projectiles[id].rot))*7)
	particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
	particlefadealpha(0.01)
	particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(projectiles[id].rot))*6,projectiles[id].y+math.cos(math.rad(projectiles[id].rot))*6)
	particlefadealpha(0.04)
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- check collision and ignite rocket
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(projectiles[id].rot))*3,projectiles[id].y-math.cos(math.rad(projectiles[id].rot))*3,1,0,1)==1 then
			-- effects
			randomseed(i+getframe()*0xABCDEF)
			size = random(3,5)
			terrainexplosion(projectiles[id].x,projectiles[id].y,size,2)
			-- ignition
			if projectiles[id].state==0 then
				-- small detonation
				arealdamage(projectiles[id].x,projectiles[id].y,30,20)
				terrainexplosion(projectiles[id].x,projectiles[id].y,10,1)
				grey=math.random(0,40)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater25,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater50,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				end
				-- arm rocket
				projectiles[id].state=1
				projectiles[id].timer=getframe() +i/msubt +weapon_timer*5
			end
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 and projectiles[id].drown==0 then
			projectiles[id].drown=1
			-- Effects
 			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile, only if not ignited
			if projectiles[id].state==0 then
				freeprojectile(id)
				break
			end
		end
		-- explode
		if projectiles[id].state==1 then
			if (getframe() +i/msubt) >= projectiles[id].timer then
				holzchopf.bunkerbuster.rocket.explode(id)
				break
			end
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end

function holzchopf.bunkerbuster.rocket.explode(id)
	arealdamage(projectiles[id].x,projectiles[id].y,120,150)
	terrainexplosion(projectiles[id].x,projectiles[id].y,100,1)
	freeprojectile(id)
end